#!/bin/bash
# Inventory System
# Author: Carsten Schoene
# $LastChangedDate: 2012-07-10 13:23:44 +0200 (Tue, 10 Jul 2012) $
# $Rev: 1540 $
#
# Default options (can be changed by inventory-system.conf)
DEBUG="yes"
TARGETURL="http://localhost/isdf.php"
ADMINMAIL="root@localhost"
#################################################################
# read optional configuration file
[ -f /etc/inventory-system.conf ] && . /etc/inventory-system.conf
#################################################################
REQUIRED_COMMANDS="
awk
cat
id
sed
grep
sort
tail
head
expr
ifconfig
ethtool
dmidecode
lshw
uname
curl
lsb_release
uuidgen
virtwhat
hostname
ip
rpm
ls
date
"

# mail alerts / debug messages
function mailalerts {
        if [ -s /tmp/inventory-system.log ] ; then
		if [ "${ADMINMAIL}" != "" ] ; then
			mail -s "Inventory System log from `hostname -f`" ${ADMINMAIL} < /tmp/inventory-system.log
		fi
	fi
}

# print debug messages to STDERR
function debug {
        if [ "${DEBUG}" == "yes" ] ; then
                echo "DEBUG: $@" >&2
		echo "DEBUG: $@" >> /tmp/inventory-system.log
        fi
}

# check required commands
for CMD in ${REQUIRED_COMMANDS} ; do
        CMDNAME=`echo ${CMD} | awk '{print toupper($1) }'`
        export $(eval "echo $CMDNAME")=`which ${CMD} 2>/dev/null`
        if [ -z "${!CMDNAME}" ] ; then
                debug "Command: ${CMD} not found!"
                exit 1
        else
                debug "Found command $(echo $CMDNAME) in ${!CMDNAME}"
        fi
done

# check if we are running as root
MYUID=`${ID} -u`
if [ "${MYUID}" != "0" ] ; then
	echo "Please start as user 'root'"
	exit 1
fi

# get unique system uuid
function get_system_uuid {
	# newer systems have uuid integrated
	UUID=`${LSHW} -quiet -C system | ${GREP} "configuration:" | ${GREP} -i uuid | ${SED} -e 's@.*[ \s]uuid=@@i'`
	# for older systems or virtual create one and save it for later use
	if [ "${UUID}" == "" ] ; then
		if [ ! -s /etc/system-inventory.uuid ] ; then
			UUID=`${UUIDGEN} -t | ${AWK} '{print toupper($1)}'`
			echo ${UUID} > /etc/system-inventory.uuid
		else
			UUID=`${CAT} /etc/system-inventory.uuid`
		fi
	fi
	echo ${UUID}
}

# read serialnumber / service tag with dmidecode
function get_system_serial {
	SYSSERIAL=`${DMIDECODE} -s system-serial-number`
	debug "SYSTEM-SERIAL: ${SYSSERIAL}"
	echo ${SYSSERIAL}
}

# get list of MAC addresses from lshw
function get_system_macs {
	${LSHW} -quiet -C network | ${GREP} serial: | ${AWK} '{print tolower($NF)}' | ${SORT} -u | \
	${GREP} -i -E '([0-9a-f]{2}:[0-9a-f]{2}:[0-9a-f]{2}:[0-9a-f]{2}:[0-9a-f]{2}:[0-9a-f]{2})'
}

# get interface name from MAC address
function get_iface_name_from_mac {
	MAC=${1}
	debug ${MAC}
	${IFCONFIG} -a | ${GREP} -i ${MAC} | ${AWK} '{print $1}' | ${HEAD} -n1
}

# get link state for interface from ethtool
function get_iface_link_state {
	IFACE=${1}
	debug ${IFACE}
	${ETHTOOL} ${IFACE} | ${GREP} -i "Link detected:" | ${AWK} '{print tolower($NF)}'
}

# get system type
function get_system_type {
	RETVAL=`${VIRTWHAT}`
	if [ $? == 0 ] ; then
		TYPE=`${VIRTWHAT} | ${TAIL} -n1`
		if [ "${TYPE}" == "" ] ; then
			TYPE="baremetal"
		fi
		debug "SYSTEM-TYOE: ${TYPE}"
		echo ${TYPE}
	else
		debug "VIRT-WHAT ERROR: ${RETVAL}"
	fi
}

# get system cpu type
function get_system_cpu {
	CPU=`${LSHW} -quiet -C processor | ${GREP} "version:" | ${SORT} -u | ${AWK} -F"version: " '{ print $NF}' | ${SED} -e 's@\s\+@ @g'`
	if [ "${CPU}" == "" ] || [ "`get_system_type`" == "vmware" ]; then
		CPU=`${LSHW} -quiet -C processor | ${GREP} "product:" | ${SORT} -u | ${AWK} -F"product: " '{ print $NF}' | ${SED} -e 's@\s\+@ @g'`
	fi
	debug "SYSTEM-CPU: ${CPU}"
	echo ${CPU}
}

# get system memory in MB
function get_system_memory {
	INSTMEM=0
	case `get_system_type` in
		xen-domU|openvz)
			INSTMEM=`${CAT} /proc/meminfo | ${GREP} "^MemTotal:" | ${AWK} '{print int($2/1024)}'`
		;;
		*)
		for MODULE in `${DMIDECODE} -t memory | ${GREP} -E '\<Size:.*MB$' | ${GREP} -iv -E '(No Module Installed|Maximum)' | ${AWK} '{print $(NF-1)}'`
		do
			INSTMEM=`${EXPR} ${INSTMEM} + ${MODULE}`
		done
		;;
	esac
	debug "SYSTEM-MEMORY: ${INSTMEM}"
	echo ${INSTMEM}
}

# get product name
function get_system_productname {
	PRODNAME=`${DMIDECODE} -s system-product-name`
	debug "SYSTEM-PRODUCTNAME: ${PRODNAME}"
	echo ${PRODNAME}
}

# get system vendor
function get_system_manufacturer {
	SYSMANUF=`${DMIDECODE} -s system-manufacturer`
	debug "SYSTEM-MANUFACTURER: ${SYSMANUF}"
	echo ${SYSMANUF}
}

# get system bios version
function get_system_biosversion {
	BIOSV=`${DMIDECODE} -s bios-version`
	debug "BIOS-VERSION: ${BIOSV}"
	echo ${BIOSV}
}

# get comma seperated list of ethernet devices (mac,iface,link)
function get_system_net_info {
        for MAC in `get_system_macs` ; do
                IF=`get_iface_name_from_mac ${MAC}`
                IFL=`get_iface_link_state ${IF}`
                echo "${MAC},${IF},${IFL}"
        done
}

# get kernel version
function get_system_kernelversion {
	KERNEL=`${UNAME} -r`
	debug "KERNEL-VERSION: ${KERNEL}"
	echo ${KERNEL}
}

# get system architecture
function get_system_arch {
	SYSARCH=`${UNAME} -m`
	debug "SYSTEM-ARCHITECTURE: ${SYSARCH}"
	echo ${SYSARCH}
	
}

# get distribution
function get_system_os {
	OSREL=`${LSB_RELEASE} -d | ${AWK} -F":\t" '{print $NF}'`
	debug "SYSTEM-OS: ${OSREL}"
	echo ${OSREL}
}

# get hostname
function get_system_hostname {
	HN=`${HOSTNAME} -f`
	debug "SYSTEM-HOSTNAME: ${HN}"
	echo ${HN}
}

# get all system ipv4 addresses
function get_system_ipv4 {
	IPV4ADDR=`${IP} -4 addr show scope global | ${GREP} inet | ${AWK} '{print $2}' | ${AWK} -F/ '{print $1}' | ${SORT} -u`
	debug "SYSTEM-IPV4-ADDRESSES: ${IPV4ADDR}"
	echo ${IPV4ADDR}
}

# get all system ipv6 addresses
function get_system_ipv6 {
	IPV6ADDR=`${IP} -6 addr show scope global | ${GREP} inet | ${AWK} '{print $2}' | ${AWK} -F/ '{print $1}' | ${SORT} -u`
	debug "SYSTEM-IPV6-ADDRESSES: ${IPV6ADDR}"
	echo ${IPV6ADDR}
}

# get primary network interface from default route
function get_system_default_interface {
	INT=`${IP} -4 route show default | ${AWK} '{print $NF}'`
	debug "SYSTEM-DEFAULT-IFACE: ${INT}"
	echo ${INT}
}

# get primary ip from interface
function get_system_default_interface_ip_from_int {
	DEVICE=${1}
	DEFAULTIP=`${IP} -4 addr show dev ${DEVICE} | ${GREP} inet | ${HEAD} -n1 | ${AWK} '{print $2}' | ${AWK} -F/ '{print $1}'`
	debug "SYSTEM-DEFAULT-IP: ${DEFAULTIP}"
	echo ${DEFAULTIP}
}

# get system default ip
function get_system_default_ip {
	get_system_default_interface_ip_from_int `get_system_default_interface`
}

# get rpm package list
function get_system_software {
	PACKAGES=`${RPM} -qa --queryformat '%{name}-%{version}-%{release}.%{arch} '`
	debug "SYSTEM-SOFTWARE: ${PACKAGES}"
	echo ${PACKAGES}
}

# get current date
function get_system_date {
	LANG=C
	DATUM=`${DATE}`
	debug "SYSTEM-DATE: ${DATE}"
	echo ${DATE}
}
	
# encode url parameters
rawurlencode() {
	local string="${1}"
	local strlen=${#string}
	local encoded=""

	for (( pos=0 ; pos<strlen ; pos++ )); do
		c=${string:$pos:1}
		case "$c" in
			[-_.~a-zA-Z0-9] )
				o="${c}"
			;;
			* )
				printf -v o '%%%02x' "'$c"
		esac
		encoded+="${o}"
	done
	echo "${encoded}"    # You can either set a return variable (FASTER) 
	REPLY="${encoded}"   #+or echo the result (EASIER)... or both... :p
}

# decode url parameters
rawurldecode() {
	printf -v REPLY '%b' "${1//%/\\x}"
	echo "${REPLY}"    # You can either set a return variable (FASTER) 
	REPLY="${decoded}"   #+or echo the result (EASIER)... or both... :p
}


# include custom functions
if [ -d /etc/inventory-system.d ] ; then
	for FILE in `${LS} -1A /etc/inventory-system.d/*.sh`
	do
		if [ -f ${FILE} ] ; then
			. ${FILE}
		fi
	done
fi

# build request
function build_request {
	request=""
	for VAR in get_system_uuid \
		get_system_serial \
		get_system_cpu \
		get_system_memory \
		get_system_manufacturer \
		get_system_productname \
		get_system_biosversion \
		get_system_net_info \
		get_system_default_ip \
		get_system_ipv4 \
		get_system_ipv6 \
		get_system_hostname \
		get_system_type \
		get_system_kernelversion \
		get_system_arch \
		get_system_os \
		get_system_software \
		get_system_date \
		${CUSTOM_FUNCTIONS} ; do
		request="${request}${VAR}="`rawurlencode "\`${VAR}\`"`
		request="${request}&"
	done
	debug ${request}
	echo ${request}
}

###### now output system data ######
if [ -n "`echo ${TARGETURL} | grep ^https`" ] ; then
	${CURL} -s -k -d "`build_request`" ${TARGETURL} >/dev/null
else
	${CURL} -s -d "`build_request`" ${TARGETURL} >/dev/null
fi
if [ $? != 0 ] ; then
	debug "Data upload failed!"
fi
mailalerts

